

subroutine CalcEnergy(Pos, M, L, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    double precision, intent(out) :: PEnergy
    real(8), parameter :: k = 1000., r0 = 0.15, da1 = 0.6, da2 = 0.4, da3 = 0.5
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d, d2, id2, id6, id12
    real(8) :: rc2, Shift, rat1, rat2
    integer :: i, j
    PEnergy = 0.
    Shift = -4. * (rc**(-12) - rc**(-6))
    rat1 = ((da2/da1)**(-6))
    rat2 = ((da2/da3)**(-6))
    rc2 = rc * rc

    do i = 0, NAtom - 1 
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            rij = Pos(j,:) - Posi
            rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
            id2 = 1. / d2            
            id6 = id2 * id2 * id2    
            id12 = id6 * id6  
            if (mod(i,M)== 0) then
                if (mod(j,M) == 0) then
                    PEnergy = PEnergy + 4.*2.5*(id12*rat1*rat1)
                              
                else if (j == i + 1 .and. mod(j, M) > 0) then
                    d = sqrt(sum(rij * rij))
                    PEnergy = PEnergy + k/2*(d - r0)**2
                    
                else
                    PEnergy = PEnergy + (4.*5.*(id12*rat2*rat2))
                end if
                    
            else if (j == i + 1 .and. mod(j, M) > 0) then
                    d = sqrt(sum(rij * rij))
                    PEnergy = PEnergy + k/2*(d - r0)**2
                    
            else if (mod(j,M) == 0) then
                    PEnergy = PEnergy + (4.*5.*(id12*rat2*rat2))
            else
                if (d2 > rc2) then
                    cycle
                end if
                if(mod(i,M) == 5 .and. mod(j,M) == 5) then
                      PEnergy = PEnergy + 4. *3.* (id12 - id6) + Shift
                else
                    PEnergy = PEnergy + 4. * (id12 - id6) + Shift
                end if
            end if
        enddo
    enddo                
end subroutine


subroutine CalcEnergyForces(Pos, M, L, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    double precision, intent(out) :: PEnergy
    double precision, intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out, inplace) :: Forces
    real(8), parameter :: k = 1000., r0 = 0.15, da1 = 0.6, da2 = 0.4, da3 = 0.5
    real(8), dimension(Dim) :: rij, Fij, Posi
    real(8) :: d, d2, id2, id6, id12
    real(8) :: rc2, Shift, rat1, rat2
    integer :: i, j
    PEnergy = 0.
    Forces = 0.
    Shift = -4. * (rc**(-12) - rc**(-6))
    rc2 = rc * rc
  
    rat1 = ((da2/da1)**(-6)) 
    rat2 = ((da2/da3)**(-6))
    
    do i = 0, NAtom - 1
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            rij = Pos(j,:) - Posi
            rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
            id2 = 1. / d2            
            id6 = id2 * id2 * id2    
            id12 = id6 * id6  
            if (mod(i,M)== 0) then
                if (mod(j,M) == 0) then
                    PEnergy = PEnergy + 4.*2.5*(id12*rat1*rat1)
                    Fij = -2.5 * rij* ((48. * id12*rat1*rat1*id2))
                              
                else if (j == i + 1 .and. mod(j, M) > 0) then
                    d = sqrt(sum(rij * rij))
                    PEnergy = PEnergy + k/2*(d - r0)**2
                    Fij = k/d * (d- r0) * rij
                    
                else
                    PEnergy = PEnergy + (4.*5.*(id12*rat2*rat2))
                    Fij = -5 * rij*((48. * id12*rat2*rat2)* id2)
                end if
                    
            else if (j == i + 1 .and. mod(j, M) > 0) then
                    d = sqrt(sum(rij * rij))
                    PEnergy = PEnergy + k/2*(d - r0)**2
                    Fij = k/d * (d- r0) * rij
                    
            else if (mod(j,M) == 0) then
                    PEnergy = PEnergy + (4.*5.*(id12*rat2*rat2))
                    Fij = -5. * rij*((48. * id12*rat2*rat2)* id2)
            else
                if (d2 > rc2) then
                    cycle
                end if
                if(mod(i,M) == 5 .and. mod(j,M) == 5) then
                    PEnergy = PEnergy + 4. * 3.*(id12 - id6) + Shift
                    Fij = rij * 3.*((-48. * id12 + 24. * id6) * id2)    
                else 
                    PEnergy = PEnergy + 4. * (id12 - id6) + Shift
                    Fij = rij * ((-48. * id12 + 24. * id6) * id2)
                end if
            end if
            Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij
        enddo
    enddo                
end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, M, L, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out,inplace) :: Pos, Vel, Accel
    double precision, intent(out) :: KEnergy, PEnergy
    real(8), dimension(Dim) :: Posi
    integer :: i
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    do i = 0, NAtom - 1
        Pos(i,:) = Pos(i,:) - L * dnint(Pos(i,:)/L)
    enddo
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, M, L, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine


subroutine Calcdistance(Pos,Count, M, L, dc, Dim, NAtom, Dim2, Nt)
    implicit none
    integer, intent(in) :: M, Dim, NAtom, Dim2, Nt
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(inout), dimension(0:Nt-1, 0:Dim2-1) :: Count
!f2py intent(in, out, inplace) :: Count
    real(8), intent(in) :: L, dc
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: dc2,d2
    integer :: i, j
    dc2 = dc * dc
    do i = 0, NAtom - 1
    Posi = Pos(i,:)
        do j = 0, NAtom - 1
            if (j .ne. i .and. mod(i,M) .eq. (M-1) .and. mod(j,M) .eq. (M-1)) then
                rij = Pos(j,:) - Posi
                rij = rij - L * dnint(rij / L)
                d2 = sum(rij * rij)
                if (d2 .le. dc2) then
                    Count(i,:) = Count(i,:) + 1
                end if
            end if
        enddo
    enddo
end subroutine
